unit qrcb;

interface

uses graphics, classes, DB, DBTables, DsgnIntf,
     qreport, qrprntr, quickrpt, qrctrls;

type
  TQRCustomCheckBox = class(TQRPrintable)
  private
    FChecked : boolean;
  protected
    procedure ReadVisible(Reader : TReader); virtual;
    procedure WriteDummy(Writer : TWriter); virtual;
  public
    procedure Paint; override;
    procedure Print(OfsX, OfsY : integer); override;
  end;

  TQRCheckBox = class(TQRCustomCheckBox)
  public
    constructor Create(AOwner : TComponent); override;
    procedure SetChecked(Value : boolean);
  published
    property Checked : boolean read FChecked write SetChecked;
  end;

  TQRDBCheckBox = class(TQRCustomCheckBox)
  private
    Field : TField;
    FieldNo : integer;
    FieldOK : boolean;
    DataSourceName : string[30];
    FDataSet : TDataSet;
    FDataField : string;
    procedure SetDataSet(Value : TDataSet);
    procedure SetDataField(Value : string);
  protected
    procedure DefineProperties(Filer: TFiler); override;
    procedure Prepare; override;
    procedure Print(OfsX, OfsY : integer); override;
    procedure ReadValues(Reader : TReader); virtual;
    procedure Unprepare; override;
    procedure WriteValues(Writer : TWriter); virtual;
  public
    constructor Create(AOwner : TComponent); override;
  published
    property DataSet : TDataSet read FDataSet write SetDataSet;
    property DataField : string read FDataField write SetDataField;
  end;

procedure Register;

implementation

procedure TQRCustomCheckBox.Paint;
begin
  with Canvas do
  begin
    Pen.Color := clBlack;
    Pen.Width := 1;
    { The original code had Width -1, Height -1.
      Which makes sense, if the box is 'n' pixels wide,
      then it should go from 0 to 'n-1'.  And the lines
      for the 'X' when checked also went to 'n-1'.  Yet,
      for some reason, it looks off a pixel.  It ain't
      too big of a deal, but it seems to look better like
      I've got it here.  But if on your system, it looks
      off a pixel, try it the other way. }

    Rectangle(0, 0, Width {-1}, Height {-1} );
    if FChecked then
    begin
      MoveTo(0, 0);
      LineTo(Width, Height - 1);
      MoveTo(0, Height - 1);
      LineTo(Width - 1, 0);
    end;
  end;
end;

procedure TQRCustomCheckBox.Print(OfsX, OfsY : integer);
var
  CalcLeft, CalcTop, CalcRight, CalcBottom : Longint;
begin
  with ParentReport.QRPrinter do
  begin
    Canvas.Pen.Color := clBlack;
    Canvas.Pen.Width := 1;
    Canvas.Brush.Style := bsClear;
    CalcLeft := XPos(OfsX + Size.Left);
    CalcTop := YPos(OfsY + Size.Top);
    CalcRight := XPos(OfsX + Size.Left + Size.Width);
    CalcBottom := YPos(OfsY + Size.Top + Size.Height);
    Canvas.Rectangle(CalcLeft, CalcTop, CalcRight, CalcBottom);
    if FChecked then
    begin
      Canvas.MoveTo(CalcLeft, CalcTop);
      Canvas.LineTo(CalcRight, CalcBottom);
      Canvas.MoveTo(CalcLeft, CalcBottom);
      Canvas.LineTo(CalcRight, CalcTop);
    end;
  end;
end;

procedure TQRCustomCheckBox.ReadVisible(Reader : TReader);
begin
  Enabled := Reader.ReadBoolean;
end;

procedure TQRCustomCheckBox.WriteDummy(Writer : TWriter);
begin
end;

constructor TQRCheckBox.Create(AOwner : TComponent);
begin
  inherited Create(AOwner);
  FChecked := False;
end;

procedure TQRCheckBox.SetChecked(Value : boolean);
begin
  if Value <> FChecked then
  begin
    FChecked := Value;
    Invalidate;
  end;
end;

constructor TQRDBCheckBox.Create(AOwner : TComponent);
begin
  inherited Create(AOwner);
  FChecked := FALSE;
  DataSourceName := '';
end;

procedure TQRDBCheckBox.DefineProperties(Filer: TFiler);
begin
  Filer.DefineProperty('DataSource',ReadValues,WriteValues,false);
  Filer.DefineProperty('Visible', ReadVisible, WriteDummy, false);
  inherited DefineProperties(Filer);
end;

procedure TQRDBCheckBox.SetDataSet(Value : TDataSet);
begin
  FDataSet := Value;
{$ifdef win32}
  if Value <> nil then
    Value.FreeNotification(self);
{$endif}
end;

procedure TQRDBCheckBox.SetDataField(Value : string);
begin
  FDataField := Value;
end;

procedure TQRDBCheckBox.Prepare;
begin
  inherited Prepare;
  if assigned(FDataSet) then
  begin
    Field := FDataSet.FindField(FDataField);
    if (Field <> nil) and (Field is TBooleanField) then
    begin
      FieldNo := Field.Index;
      FieldOK := true;
    end else
    begin
      Field := nil;
      FieldOK := false;
    end;
  end else
  begin
    Field := nil;
    FieldOK := false;
  end;
end;

procedure TQRDBCheckBox.Unprepare;
begin
  Field := nil;
  inherited Unprepare;
  if DataField <> '' then
    SetDataField(DataField) { Reset component caption }
  else
    SetDataField(Name);
end;

procedure TQRDBCheckBox.ReadValues(Reader : TReader);
begin
  DataSourceName := Reader.ReadIdent;
end;

procedure TQRDBCheckBox.WriteValues(Writer : TWriter);
begin
end;

procedure TQRDBCheckBox.Print(OfsX, OfsY : integer);
begin
  if FieldOK then
  begin
    if FDataSet.DefaultFields then
      Field := FDataSet.Fields[FieldNo];
  end
  else
    Field := nil;

  FChecked := FALSE;
  if assigned(Field) then
    if (Field is TBooleanField) then
      FChecked := TBooleanField(Field).value;

  inherited Print(OfsX,OfsY);
end;

procedure Register;
begin
  RegisterComponents('QReport', [TQRCheckBox, TQRDbCheckBox]);

  {If you get an error on the statement below,
  TQRDataFieldProperty unknown identifier, then
  you need to do the following edits to QREPORT.PAS:

  1.  Move the definitions for TQRDBStringProperty &
      TQRDataFieldProperty to the interface section.

  2.  Change the uses statement for the interface section to:
      "uses Classes, DsgnIntf, QuickRpt;"

  3.  Remove the Classes and DsgnIntf references from the
      uses statements in the implementation section
      (both the Win32 and the non-Win32 versions)

  If you write additional QuickReport componenets, (or even
  non-QuickReport components) the QR property editors come in
  handy.  You may want to make other property editors public also.
  }
  RegisterPropertyEditor( TypeInfo(string), TQRDBCheckBox, 'DataField',
                          TQRDataFieldProperty );


{$ifdef ver100}
  {I have no clue what this means.  Does "ver100" mean Delphi 1?
  I don't think so.  Maybe C++ Builder 1? I just followed the
  example in QREPORT.PAS }
  RegisterNonActiveX([TQRCheckBox, TQRDbCheckBox]);
{$endif}

end;

end.

